*************************************************************
*************************************************************
****                                                     ****
****       c l a v i e r        t  o    9 +              ****
****                                                     ****
*************************************************************
*************************************************************
*                                                           *
* microprocesseurs   6805 p2, ram  64, rom 1100, 28 broches *
*                   68705 p3, ram 112, rom 1804, 28 broches *
*
*************************************************************
*                                                           *
* version du :  24  janvier  1986         *   fichier clo   *
*                                                           *
* pour masquage
*************************************************************
*
* adresses programme en rom
prog   equ    $3c0     ;6805 p2, fin rom en $783
*prog   equ    $100     ;68705 p3, fin rom en $783
*prog   equ    $100     ;68705 u2, fin de rom en $f37
tab1   equ    $80      ;rom en page 0
*************************************************************
* constantes timer ( pre-diviseur /32 )
tcron  equ    $15      ;interruption timer autorisee
tcroff equ    $55      ;interruption timer interdite
tcrz   equ    $5d      ;pre-diviseur a zero
mor    equ    $45      ;mask option register sur 68705
**********************
* constantes diverses
tcaps  equ    48       ;position dans la matrice
*************************************************************
* memoire ram          ( 32 octets disponibles )
       org    $40      ;fin ram en $5f (pile de $60 a $7f)
dram   equ    *
fram   equ    $5f
*
exp    rmb    1        ;type du clavier, francais=0, export<>0
code1  rmb    1        ;code apres scrutation matrice
code2  rmb    1
code11 rmb    1
codef  rmb    1        ;code final a envoyer
col    rmb    1        ;colonne de la matrice
col2   rmb    1
minmaj rmb    1        ;code minuscule/majuscule et peripherique
mm     rmb    1        ;indicateur min/maj, sauvegarde port b
mmp    rmb    1        ;valeur precedente de minmaj
hex    rmb    1        ;temporaire en mode hexa
temp   rmb    1        ;temporaire (scr)
temp2  rmb    1        ;temporaire (scr)
compt  rmb    1        ;compteur des 8 bits, sortie serie
c1000  rmb    1        ;constante d'attente repetition
c70    rmb    1        ;constante de cadence repetition
crep1  rmb    1        ;compteur delai repetition 0.8 seconde
crep70 rmb    1        ;compteur delai repetition 70 milli-seconde
sa     rmb    1        ;sauvegarde a pour temporisations
pcc    rmb    1        ;sauvegarde du port c ( rotations )
*************************************************************
*  indicateurs
indic  rmb    1        ;bit 7= autorisation d'envoi du code clavier
*                      bit 6= une touche valide pressee
*                      bit 5= touche verrouillage pressee
*                      bit 4= mode hexa-decimal
*                      bit 3=
*                      bit 2= code 1 envoye
*                      bit 1= deux touches detectees (scr)
*                      bit 0= une touche detectee (scr)
*
mode   rmb    1        ;bit 5= mode vt100
*                      bit 0= min en shift caps
*
sc     rmb    1        ;bit 1= touche shift enfoncee
*                      bit 0= touche controle enfoncee
*************************************************************
* entrees/sorties
       org    0
pa     rmb    1
pb     rmb    1
pc     rmb    1
       rmb    1
clav   equ    pa
dira   rmb    1
dirb   rmb    1
dirc   rmb    1
       rmb    1
timer  rmb    1
tcr    rmb    1
       rmb    1
pcreg  rmb    1        ;(sur 68705 uniquement)
*************************************************************
* port a : entree des 8 lignes de la matrice des touches
*               a7    = selection francais/export au reset
***************
* port b : sorties vers les lignes de la matrice (l1 a l8)
*               b7    = commande de la led  caps
*               b6    = entre touche shift
*               b5    = entree touche controle
*               b2    = entree francais/export
led    equ    7
shift  equ    6
cntrl  equ    5
fraexp equ    2
***************
* port c :      c0,c1 = scrutation de la matrice ( l9, l10 )
*               c1    = sortie donnee serie
*               c2    = sortie ktest
*               c3    = sortie horloge (liaison serie synchrone)
sd     equ    1
kt     equ    2
h      equ    3
*************************************************************
* mask option
*       org    $f38     68705 u3 (ou r3)
*      org    $784     68705 p3
*      fcb    mor
*************************************************************
*pointeurs
*       org    $ff8     68705 u3, 68 p 05 v 07
       org    $7f8     ;6805 p2, 68705 p3
       fdb    inttim   ;interruption timer
       fdb    int      ;interruption externe
       fdb    init     ;swi (non utilise)
       fdb    init     ;reset
*************************************************************
* fonctions speciales
*  obtenues par appui sur les touches cntrl et shift
*  puis appui momentane sur la touche caps
*  et enfin appui sur la touche selon la fonction desiree
*
*  touche    f : clavier francais
*            e : clavier export
*            v : pave numerique vt100
*            n : pave numerique normal
*            m : minuscules en shift caps
*          raz : minuscules normales
*    fonctions : attente repetition (0,4 a 1,6 secondes )
*   ligne num. : cadence repetition (10 a 100 ms )
*    pave num. : commandes unite centrale (0 a 5 )
*************************************************************
* fonction hexa-decimale
* envoi d'un code quelconque compris entre $00 et $ff
*  obtenue par appui sur la touche cntrl
*  puis appui momentane sur la touche caps
*  et enfin appui sur les 2 touches hexa-decimales
*  ( touches 0...9 du pave, 0...9 a,b,c,d,e,f )
*************************************************************
*************************************************************
*  clavier to 9+  ***  24 janvier 1986  ***  fichier cl1  ***
*************************************************************
       org    prog
*
* mise a zero de toute la ram
init   ldx    #dram
in2    clr    0,x
       incx
       cpx    #fram+1
       bne    in2
* initialisation des ports
       lda    #$ff
       sta    dirb      ;port b= sorties
       lda    #$0f      ;port c= sorties ( c0 a c3 )
       sta    dirc
       lda    #$7f
       sta    pb
       sta    mm
       lda    #$03
       sta    pc
       lda    #$b4      ;code majuscule
       sta    minmaj
       lda    #80
       sta    c1000
       lda    #7
       sta    c70
*************************************************************
debut  lda    #tcroff    ;int. timer interdites
       sta    tcr
       lda    minmaj
       cmp    mmp      ;valeur precedente de minmaj
       beq    debut2
       sta    mmp
       bsr    sortie    ;seulement si minmaj a change
       bsr    t4ms
debut2 jsr    inirep
*************************************************************
debut3 lda    #$ff
       sta    timer
* attente d'interruption timer
att    rsp
       bclr   7,tcr    ;indicateur int timer = 0
       bclr   6,tcr    ;int timer autorisees
       cli
       bsr    horlog   ;impulsion d'horloge
att1   bsr    t320
       bra    att1
*************************************************************
ktest  sta    sa       ;( duree = 50 cycles )
       ldx    pb
       lda    #$ff
       sta    pb       ;toutes les lignes = 1
       lda    pa
       stx    pb       ;restaure immediatement la led caps
       tsta
       beq    kt1
       bset   kt,pc    ;ktest=1
       lda    sa
       rts
kt1    bclr   kt,pc    ;ktest=0
       lda    sa
       rts
*************************************************************
* boucle principale (inetrruption timer)
inttim bclr   7,tcr          ;raz indicateur int timer
       bset   6,tcr          ;pas d'int. timer
       cli                   ;permet la reception serie
       jsr    scrclv         ;scrutation clavier
       brclr  7,indic,att    ;indic code a envoyer
       bsr    sortic         ;code envoye
       bra    att
*************************************************************
* sortie du code en serie  (le code est dans a)
sortic lda    codef
sortie sei
       ldx    #8       ;compteur des 8 bits
       stx    compt
*start bit
       bclr   sd,pc
       bsr    tbit     ;(8+39 micro-seconde)
* bits de donnees
sr1    lsra
       bcs    sr2
       bclr   sd,pc    ;bit = 0
       bra    sr3
sr2    bset   sd,pc    ;bit = 1
sr3    bsr    tbit
       dec    compt
       bne    sr1
* stops bits (3)
       bset   sd,pc
       bsr    tbit
       bsr    tbit
       bclr   7,indic      ;code envoye
       cli
       rts
*************************************************************
* temporisations
t320   sta    sa       ;tempo 320 micro-seconde
       lda    #5
       bra    ttt
t4ms   sta    sa       ;tempo 4 milli-seconde
       lda    #68
ttt    bsr    ktest    ;boucle 55 cycles
       deca
       bne    ttt
       lda    sa
       rts
*************************************************************
tbit   jsr    ktest
horlog bset   h,pc     ;horloge=1
       tsta
       tsta
       bclr   h,pc     ;horloge=0
       rts
*************************************************************
* reception d'un code serie de 3 bits sur interruption int
int    bset   6,tcr    ;interdiction interruption timer
       clra
*verification du bit de start
       bih    intfin   ;test bit de start
       bsr    tbit     ;tempo 47 micro-s
*prise en compte du bit 0
       bil    int3
       ora    #1
int3   bsr    tbit
*prise en compte du bit 1
       bil    int4
       ora    #2
int4   bsr    tbit
*prise en compte du bit 2
       bil    int5
       ora    #4
int5   bsr    tbit
       bsr    tbit
       bsr    tbit
       bsr    tbit
       bsr    tbit
       bsr    tbit
*verification du stop bit
       bih    cmd
intfin jmp    debut
*************************************************************
cmd    tax
cmd2   ldx    saut,x
       jmp    rst,x
*
saut   fcb    rst-rst  ;reset logiciel
       fcb    vmaj-rst ;verrouillage majuscule
       fcb    vmin-rst ;retour en minuscule
       fcb    vt-rst   ;vt100
       fcb    nvt-rst  ;non vt100
       fcb    type-rst ;demande de type min/maj
********************
rst    bsr    t4ms     ;reset logiciel
       jmp    init
*
vmaj   jsr    cps1     ;verrouillage majuscule
       bra    intfin
*
vmin   jsr    cps2     ;retour en minuscule
       bra    intfin
*
vt     bset   5,mode
       bra    intfin
*
nvt    bclr   5,mode
       bra    intfin
*
type   clr    mmp
       bsr    t4ms
       jmp    debut
*************************************************************
*************************************************************
*  clavier to 9+  ***  21 janvier 1986  ***  fichier cl2  ***
*************************************************************
* scrutation du clavier *
scrclv jsr    scr
       bcs    clvfin
       brclr  6,indic,sc1  ;saut si premier appui
       cmp    code11
       beq    sc2
*************************************************************
* retour scrutation
clvfin clr    indic
       bclr   kt,pc    ;ktest=0
inirep lda    c1000    ;init attente repetition
       sta    crep1
inicad lda    c70      ;init cadence repetition
       sta    crep70
       rts
*************************************************************
* deuxieme scrutation si premier appui
sc1    sta    code11
       jsr    t320
       jsr    scr
       bcs    clvfin
       cmp    code11
       bne    inirep
*************************************************************
sc2    cmp    #tcaps   ;test touche verrou
       bne    rech
       jmp    tverr
*************************************************************
* recherche du code dans les tableaux
rech   lda    $80,x    ;lecture du tableau 1
       brclr  5,mode,rech1
******
       cmp    #11      ;mode vt100
       bhi    rech1
       add    #$9a
       bra    cnt2
******
rech1  brclr  0,exp,rech2  ;saut si clavier francais
* clavier export
       brclr  1,sc,rech2   ;saut si shift non pressee
       cmp    #13          ;touche acc
       bne    re0
       lda    #$40     ;@
       bra    cnt1
re0    cmp    #21          ;touche stop
       bne    rech2
       lda    #$5c     ;\
       bra    cnt1
******
rech2  cmp    #20          ;touches communes (1 octet)
       bhi    rech3
       tax
       lda    tabcom,x
       bra    cnt2
******
rech3  sub    #21
       lsla
       cmp    #7*2
       bls    noninv
       cmp    #27*2
       bls    tstver
       brset  0,exp,export
* clavier francais
       cmp    #37*2
       bls    noninv
       cmp    #43*2
       bhi    noninv
       bra    tstver
* clavier export
export add    #29*2
       cmp    #66*2
       bls    noninv
       cmp    #75*2
       bhi    noninv
tstver brset  7,mm,noninv      ;saut si non verrouillage
       brclr  0,mode,colmaj    ;saut en mode normal et verrouillage
       brclr  1,sc,colmaj      ;shift au repos = maj
       bra    colmin           ;shift actif = min
noninv brclr  1,sc,colmin
colmaj inca
colmin tax
       lda    tab2,x
cnt1   brclr  0,sc,cnt2        ;test de la touche controle
       cmp    #$3f
       bls    cnt2
       cmp    #$7f
       bhi    cnt2
       and    #%00011111
******
cnt2   sta    codef
       bset   6,indic         ;touche pressee
       brset  2,indic,attrep  ;repetition si code deja envoye
       bset   7,indic         ;autorisation d'envoi
       bset   2,indic         ;code envoyé
       rts
*************************************************************
* repetition automatique
attrep tst    crep1
       beq    repet
       dec    crep1
repfin rts
repet  dec    crep70
       bne    repfin
       bset   7,indic
       jmp    inicad
*************************************************************
* touche verrouillage
tverr  brset   0,sc,hexa    ;saut si touche cntl enfoncee
*************************************************************
* action sur la touche verrouillage
cps    brset  5,indic,cpsf   ;touche deja enfoncee
       bsr    cpss
       sta    codef
       bset   7,indic   ;autorisation de sortie du code
       bset   5,indic   ;touche caps enfoncee
cpsf   rts
******
cpss   tst    mm
       bpl    cps2
cps1   lda    #$b4     ;passage en majuscules
       ldx    #$7f
       bra    cps3
cps2   lda    #$b0     ;passage en minuscules
       ldx    #$ff
cps3   sta    minmaj
       sta    mmp
       stx    mm
       stx    pb
       rts
*************************************************************
hexa   bset   6,tcr    ;pas d'interruption timer
       brclr  1,sc,hexa2
* fonctions speciales
       bsr    attrel
h1     jsr    scrt     ;attente de frappe d'une touche
       bcs    h1
       ldx    $80,x
*
       cpx    #5
       bhi    h01
       stx    hex
       bsr    attrel
       ldx    hex
       jmp    cmd2     ;fonctions unite centrale
h01    cpx    #28
       bhi    h02
       txa
       sub    #24
       bmi    h02
       lsla
       lsla
       lsla
       lsla
       lsla            ;a=a*32
       add    #40
       sta    c1000    ;attente repetition variable
       bra    h9
h02    cpx    #46      ;touche v, clavier vt100
       bne    h4
       bset   5,mode
h4     cpx    #48      ;touche n, clavier non vt100
       bne    h6
       bclr   5,mode
h6     cpx    #63      ;touche m, min en shift caps
       bne    h7
       bset   0,mode
h7     cpx    #23      ;touche raz, annulation min
       bne    h8
       bclr   0,mode
h8     cpx    #58
       bhi    h9
       txa
       sub    #49
       bmi    h9
       lsla
       inca
       sta    c70      ;cadence repetition variable
h9     bra    sph3
*************************************************************
* fonction hexa-decimale
hexa2  bset   4,indic  ;fonction hexa active
hx1    bsr    attrel   ;attente relachement de la touche caps
       bsr    sph      ;entree du 1 caractere hexa
       lsla
       lsla
       lsla
       lsla
       sta    hex
       bsr    attrel   ;attente relachement de la touche hexa
       bsr    sph      ;entree du 2 caractere hexa
       add    hex
       jsr    sortie
       bra    hx1
**********************
attrel bsr    scrt     ;attente de relachement d'une touche
       bcc    attrel
       rts
**********************
sph    brclr  0,sc,hxf    ;touche cntrl relachee
       bsr    scrt
       bcs    sph      ;attente frappe du caractere hexa
       ldx    #tabhex
sph1   cmp    0,x
       bne    sph2
       txa
       and    #$0f
       rts
sph2   incx
       cpx    #tabhex+27
       bne    sph1
sph3   bsr    attrel   ;touche non valide,attente relachement
hxf    bclr   4,indic
hxfin  jmp    debut
*************************************************************
* scrutation de la matrice des touches *
*   10 lignes (entrees) et 8 colonnes (sorties)
scrt   jsr    t4ms
scr    bclr   0,indic           ;1 t detectee
       bclr   1,indic           ;2 t detectees
       clr    code2
****** test des touches shift et controle
       clr    sc
       clra                     ;port b = entree
       sta    dirb
       tsta                     ;* tempo
       tsta                     ;*
       brclr  shift,pb,sc01
       bset   1,sc              ;indicateur shift
sc01   brclr  cntrl,pb,sc02
       bset   0,sc              ;indicateur controle
sc02   brclr  fraexp,pb,sc03    
       deca
sc03   sta    exp               ;lecture type de clavier
       lda    #$ff
       sta    dirb              ;port b en sortie
******
       sta    pb
       lda    pc
       ora    #$03               
       sta    pc       ;met les 10 lignes a 1
       sta    pcc
anti   lda    clav
       bne    ant
       lda    mm
       sta    pb       ;restaure le voyant
scerr  sec
       rts
ant    sta    col
       jsr    t320     ;* anti-rebonds
       lda    clav     ;*
       cmp    col      ;*
       bne    anti     ;*
*
       lda    #$fe     ;met la premiere ligne a 0
       sta    pb
       clrx            ;compteur de lignes ( 0 a 9 )
* boucle executee 10 fois
scr1   lda    clav
       eor    col      ;a#0 si une colonne est repassee a 0
       bne    scr3
scr2   cpx    #8
       bhi    scfin
       incx
       sec
       rol    pb
       rol    pcc
       lda    pcc
       and    #7
       sta    pc
       bra    scr1
* colonne trouvee. calcul du numero de la matrice (de 0 a 79)
scr3   sta    col2     ;bit(s) de a a 1 indique colonne(s)
       brset  1,indic,scr2  ;interdit plus de 2 touches
       txa
       lsla            ;a = a * 8
       lsla
       lsla
       sta    temp
       lda    #$ff     ;calcul du numero de la colonne (0 a 7)
scr4   inca            ;de la premiere touche
       lsr    col2
       bcc    scr4
       sta    temp2
       add    temp
       brset  0,indic,scr5  ;une touche deja detectee
       sta    code1
       bset   0,indic  ;une touche deja detectee
       bra    scr2
scr5   bset   1,indic  ;deux touches detectees
       bra    scr2
**********************
scfin  lda    mm
       sta    pb               ;restaure le voyant
       lda    pc
       ora    #3
       sta    pc               ;remet les 10 lignes a l'etat haut
       brset  1,indic,scerr    ;erreur si 2 touches pressees
       lda    code1
       tax
       clc
       rts
*************************************************************
xxxx   equ    *
*************************************************************
***********************************************************************
* clavier to 9+ *** 20 janvier 1986 *** fichier tab *******************
***********************************************************************
*************     tableaux de matricage du clavier     ****************
***********************************************************************
*
* premier tableau du matricage clavier to 9+
*
*-----------------------------------------
* 10 lignes : 0  1  2  3  4  5  6  7  8  9
*            b2 b1 b0 c1 c0 b3 b4 b5 b6 b7
*-----------------------------------------
       org    tab1
*
*  port a     a0 a1 a2 a3 a4 a5 a6 a7
*  ------     -----------------------
       fcb    27,52,30,38,13,01,22,46    ;f4  4  r f acc p1 del v
       fcb    26,53,31,39,68,17,19,47    ;f3  5  t  g =+  - ins b
       fcb    25,54,61,40,15,18,23,48    ;f2  6  y  h  !  - raz n
       fcb    72,49,59,62,71,05,06,64     ;@  1  a  q ([ p5 p6  w
       fcb    21,55,32,41,14,09,11,66  ;stop  7  u  j sp p9 pe ,?
       fcb    00,56,33,42,70,16,73,75        ;8  i  k $&  ! }] .;
       fcb    20,57,34,43,69,65,12,76  ;caps  9  o  l  - %u ent :/
       fcb    00,58,35,63,74,67,03,77        ;0  p  m  )  ^ p3 <>
       fcb    24,50,60,36,08,02,10,44    ;f1  2  z  s p8 p2 p.  x
       fcb    28,51,29,37,07,04,00,45    ;f5  3  e  d p7 p4 p0  c
************************************************************************
* tables des touches hexa-decimales
tabhex fcb    57,25,65,73,01,09,17,33,41    ;0  1  2  3  4  5  6  7  8
       fcb    49,26,15,79,75,74,03,78,05    ;9  a  b  c  d  e  f p0 p1
       fcb    69,62,77,29,30,76,68,37,27   ;p2 p3 p4 p5 p6 p7 p8 p9  q
***********************************************************************
* table des touches communes ( 1 seul octet )
tabcom fcb    $30,$31,$32,$33,$34,$35,$36    ;p0  p1  p2  p3  p4  p5  p6
       fcb    $37,$38,$39,$2e,$0d,$0d,$16    ;p7  p8  p9  p.  pe ent acc
       fcb    $20,$0b,$0a,$08,$09,$1c,$00    ;esp fl.h fl.b fl.g fl.d ins
***********************************************************************
* table de conversion des codes clavier (2)
       org    xxxx
* touches communes
tab2   fcb    $02,$03  ;stop,break
       fcb    $1d,$7f  ;eff,del
       fcb    $1e,$0c  ;home,raz
       fcb    $90,$95  ;fonction 1
       fcb    $91,$96  ;fonction 2
       fcb    $92,$97  ;fonction 3
       fcb    $93,$98  ;fonction 4
       fcb    $94,$99  ;fonction 5
*********************************************
* touches identiques sur les 2 claviers
*         ( inversion en verrouillage )
       fcb    $65,$45  ;e
       fcb    $72,$52  ;r
       fcb    $74,$54  ;t
       fcb    $75,$55  ;u
       fcb    $69,$49  ;i
       fcb    $6f,$4f  ;o
       fcb    $70,$50  ;p
       fcb    $73,$53  ;s
       fcb    $64,$44  ;d
       fcb    $66,$46  ;f
       fcb    $67,$47  ;g
       fcb    $68,$48  ;h
       fcb    $6a,$4a  ;j
       fcb    $6b,$4b  ;k
       fcb    $6c,$4c  ;l
       fcb    $78,$58  ;x
       fcb    $63,$43  ;c
       fcb    $76,$56  ;v
       fcb    $62,$42  ;b
       fcb    $6e,$4e  ;n
*********************************************
*touches specifiques au clavier francais
*
*         ( pas d'inversion )
       fcb    $2a,$31  ;* 1
       fcb    $80,$32  ;e accent aigu  2
       fcb    $22,$33  ;" 3
       fcb    $27,$34  ;/ 4
       fcb    $28,$35  ;( 5
       fcb    $5f,$36  ;_ 6
       fcb    $81,$37  ;e accent grave  7
       fcb    $21,$38  ;! 8
       fcb    $82,$39  ;c cedille  9
       fcb    $83,$30  ;a accent  0
*
*         ( inversion en verrouillage )
       fcb    $61,$41  ;a
       fcb    $7a,$5a  ;z
       fcb    $79,$59  ;y
       fcb    $71,$51  ;q
       fcb    $6d,$4d  ;m
       fcb    $77,$57  ;w
*
*         ( pas d'inversion )
       fcb    $84,$25  ;u accent   %
       fcb    $2c,$3f  ;, ?
       fcb    $5e,$85  ;^ trema
       fcb    $3d,$2b  ;= +
       fcb    $2d,$5c  ;- \
       fcb    $24,$26  ;$ &
       fcb    $5b,$7b  ;[ {
       fcb    $23,$40  ;# @
       fcb    $5d,$7d  ;] }
       fcb    $29,$86  ;) numero
       fcb    $3b,$2e  ;; .
       fcb    $3a,$2f  ;: /
       fcb    $3e,$3c  ;> <
*********************************************
* touches specifiques au clavier export
*
*         ( pas d'inversion )
       fcb    $31,$21  ;1 !
       fcb    $32,$22  ;2 "
       fcb    $33,$87  ;3 paragraphe
       fcb    $34,$24  ;4 $
       fcb    $35,$25  ;5 %
       fcb    $36,$26  ;6 &
       fcb    $37,$2f  ;7 /
       fcb    $38,$28  ;8 (
       fcb    $39,$29  ;9 )
       fcb    $30,$3d  ;0 =
*
*         ( inversion en verrouillage )
       fcb    $71,$51  ;q
       fcb    $77,$57  ;w
       fcb    $79,$59  ;y
       fcb    $61,$41  ;a
       fcb    $8d,$8a  ;o trema
       fcb    $7a,$5a  ;z
       fcb    $8c,$89  ;a trema
       fcb    $6d,$4d  ;m
       fcb    $8e,$8b  ;u trema
*
*         ( pas d'inversion )
       fcb    $5d,$7d  ;] )
       fcb    $27,$8f  ;' accent grave
       fcb    $2b,$2a  ;+ *
       fcb    $3e,$3c  ;> <
       fcb    $5b,$7b  ;[ (
       fcb    $23,$5e  ;# ^
       fcb    $88,$3f  ;beta  ?
       fcb    $2c,$3b  ;, ;
       fcb    $2e,$3a  ;. :
       fcb    $2d,$5f  ;- _
***********************************************************************
       end
